@@ -5,7 +5,8 @@ loadModule("xmap_module", TRUE)
 .onAttach <- function(...) {
     if (!interactive()) return()
     
-    intro_message <- paste("If you're new to the rEDM package, please check out the tutorial:",
-                           "> vignette(\"rEDM_tutorial\")", sep = "\n")
+    intro_message <- paste("If you're new to the rEDM package, please check out 
+                           the tutorial:",
+                           "> vignette(\"rEDM-tutorial\")", sep = "\n")
     packageStartupMessage(intro_message)
 }
\ No newline at end of file
