@@ -165,19 +165,6 @@ void BlockLNLP::run()
 }
 
 DataFrame BlockLNLP::get_output()
-{
-    return DataFrame::create( Named("time") = target_time, 
-                              Named("obs") = targets, 
-                              Named("pred") = predicted, 
-                              Named("pred_var") = predicted_var);
-}
-
-List BlockLNLP::get_smap_coefficients()
-{     
-    return(wrap(smap_coefficients));
-}
-
-DataFrame BlockLNLP::get_short_output()
 {
     vec short_time(which_pred.size(), qnan);
     vec short_obs(which_pred.size(), qnan);
@@ -198,6 +185,28 @@ DataFrame BlockLNLP::get_short_output()
                               Named("pred_var") = short_pred_var);
 }
 
+DataFrame BlockLNLP::get_smap_coefficients()
+{
+    size_t embed_dim = smap_coefficients.size();
+    List tmp_lst(embed_dim);
+    CharacterVector df_names(embed_dim);
+    vec temp_coeff;
+    for(size_t j = 0; j < embed_dim; ++j)
+    {
+        temp_coeff.assign(which_pred.size(), qnan);
+        for(size_t i = 0; i < which_pred.size(); ++i)
+        {
+            temp_coeff[i] = smap_coefficients[j][which_pred[i]];
+        }
+        tmp_lst[j] = temp_coeff;
+        df_names[j] = "c_" + std::to_string(j+1);
+    }
+    df_names[embed_dim - 1] = "c_0";
+    DataFrame df(tmp_lst);
+    df.attr("names") = df_names;
+    return(df);
+}
+
 DataFrame BlockLNLP::get_stats()
 {
     PredStats output = make_stats();
@@ -315,7 +324,6 @@ RCPP_MODULE(block_lnlp_module)
     .method("run", &BlockLNLP::run)
     .method("get_output", &BlockLNLP::get_output)
     .method("get_smap_coefficients", &BlockLNLP::get_smap_coefficients)
-    .method("get_short_output", &BlockLNLP::get_short_output)
     .method("get_stats", &BlockLNLP::get_stats)
     ;
 }
